<?php

namespace common\modules\orderPayment;

use Axepta\Payment\CartData;
use Axepta\Payment\Notification;
use Axepta\Payment\Paywall;
use Axepta\Payment\Util;
use common\classes\Currencies;
use common\classes\extended\OrderAbstract;
use common\classes\modules\ModulePayment;
use common\classes\modules\ModuleStatus;
use common\classes\modules\ModuleSortOrder;
use common\helpers\Order;
use common\helpers\OrderPayment;
use common\models\OrdersPayment;
use Yii;

require 'lib/axepta/payment-core/vendor/autoload.php';

/**
 * class declaration
 */
class axepta extends ModulePayment
{

	/**
	 * variables
	 */
	var $code, $title, $description, $enabled, $version, $order_id;

	/**
	 * default values for translation
	 */
	protected $defaultTranslationArray = [
		'MODULE_PAYMENT_AXEPTA_TEXT_TITLE'       => 'Axepta',
		'MODULE_PAYMENT_AXEPTA_TEXT_REDIRECT'    => 'Please wait, in the next step you will be redirected to paywall',
		'MODULE_PAYMENT_AXEPTA_TEXT_DESCRIPTION' => 'Online payment methods, for example BLIK, Pay-By-Link',
		'MODULE_PAYMENT_AXEPTA_ERROR'            => 'There has been an error processing payment',
	];

	/**
	 * class constructor
	 */
	function __construct()
	{
		parent::__construct();

		$this->code = 'axepta';
		$this->version = '1.0.0';
		$this->title = MODULE_PAYMENT_AXEPTA_TEXT_TITLE;
		$this->description = MODULE_PAYMENT_AXEPTA_TEXT_DESCRIPTION;
		if(!defined('MODULE_PAYMENT_AXEPTA_STATUS')) {
			$this->enabled = false;

			return false;
		}
		$this->sort_order = MODULE_PAYMENT_AXEPTA_SORT_ORDER;
		$this->enabled = MODULE_PAYMENT_AXEPTA_STATUS == 'True';
		$this->online = false;

		if((int) MODULE_PAYMENT_AXEPTA_ORDER_STATUS_ID > 0) {
			$this->order_status = MODULE_PAYMENT_AXEPTA_ORDER_STATUS_ID;
		}

		$this->update_status();
	}

	function update_status()
	{

		if($this->enabled && ((int) MODULE_PAYMENT_AXEPTA_ZONE > 0)) {
			$check_flag = false;
			$check_query = tep_db_query("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_AXEPTA_ZONE . "' and zone_country_id = '" . $this->delivery['country']['id'] . "' order by zone_id");
			while($check = tep_db_fetch_array($check_query)) {
				if($check['zone_id'] < 1) {
					$check_flag = true;
					break;
				} elseif($check['zone_id'] == $this->delivery['zone_id']) {
					$check_flag = true;
					break;
				}
			}

			if(!$check_flag) {
				$this->enabled = false;
			}
		}
	}

	function selection()
	{
		return [
			'id'     => $this->code,
			'module' => $this->title,
		];
	}

	public static function getVersionHistory()
	{
		return [
			'1.0.0' => 'init module',
		];
	}

	function process_button()
	{
		return false;
	}

	function after_process()
	{
		$this->process_payment_link();
	}

	function getItemsArray($order)
	{
		$items = [];
		foreach($order->products as $orderItem) {
			$items[] = [
				'name'   => $orderItem['name'],
				'amount' => Util::convertAmountToFractional($orderItem['final_price'] * $orderItem['qty']),
				//TODO: check if need currencyValue here
				'qty'    => (integer) $orderItem['qty'],
				'price'  => $orderItem['final_price'],
			];
		}

		$order_details = $order->info;
		$items[] = [
			'name'   => $order_details['shipping_method'],
			'amount' => $order_details['shipping_cost'],
			'qty'    => 1,
			'price'  => $order_details['shipping_cost'],
		];

		if($order_details['tax'] > 0) {
			$items[] = [
				'name'   => 'Tax',
				'amount' => $order_details['tax'],
				'qty'    => 1,
				'price'  => $order_details['tax'],
			];
		}

		$discount_amount = $_SESSION['order_summary']['credits_applied'];
		if($discount_amount > 0) {
			$items[] = [
				'name'   => 'Round',
				'amount' => -$discount_amount,
				'qty'    => 1,
				'price'  => -$discount_amount,
			];
		}

		return $items;
	}

	function process_payment_link()
	{

		$currencies = Yii::$container->get('currencies');
		$order = $this->manager->getOrderInstance();

		$amount = Util::convertAmountToFractional(
			round($currencies->format_clear($order->info['total_inc_tax'], true, $order->info['currency']), 2)
		);

		$cart_data = new CartData();
		$cart_data->setGrandTotal($amount);
		$cart_data->setAddressBilling(
			$order->billing['city'],
			$order->billing['street_address'],
			$order->billing['country']['title'],
			$order->billing['postcode']
		);
		$cart_data->setAddressDelivery(
			$order->delivery['city'],
			$order->delivery['street_address'],
			$order->delivery['country']['title'],
			$order->delivery['postcode']
		);

		$currency_value = (float) $order->info['currency_value'];

		foreach($this->getItemsArray($order) as $key => $value) {

			$cart_data->addItem(
				$value['name'],
				Util::convertAmountToFractional($value['price'] * $currency_value),
				$value['qty']
			);
		}

		$paywall = new Paywall(
			Util::ENVIRONMENT_PRODUCTION,
			MODULE_PAYMENT_AXEPTA_SERVICE_KEY,
			MODULE_PAYMENT_AXEPTA_SERVICE_ID,
			MODULE_PAYMENT_AXEPTA_MERCHANT_ID
		);

		$form = $paywall->buildOrderForm(
			Paywall::prepareData(
				$amount,
				$order->info['currency'],
				$order->order_id,
				$order->customer['firstname'],
				$order->customer['lastname'],
				$order->customer['email_address'],
				$order->customer['phone'],
				tep_href_link(FILENAME_CHECKOUT_SUCCESS, '', 'SSL'),
				tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&' . tep_session_name() . '=' . tep_session_id(), 'SSL', false),
				tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&' . tep_session_name() . '=' . tep_session_id(), 'SSL', false),
				MODULE_PAYMENT_AXEPTA_CREDIT_TYPE,
				$cart_data->prepareCartData(),
				$this->version . ';oscommerce_' . PROJECT_VERSION_MAJOR . '.' . PROJECT_VERSION_MINOR . '.' . PROJECT_VERSION_PATCH,
				tep_href_link('callback/webhooks.payment.' . $this->code, 'action=notification&order_id=' . $order->order_id, 'SSL')
			)
		);

		if(!$form) {
			return false;
		}

		global $cart;
		$cart->reset(true);

		echo MODULE_PAYMENT_AXEPTA_TEXT_REDIRECT
			. '<br/>'
			. $form
			. '<script type="text/javascript">'
			. 'document.getElementById(\'submit-payment-form\').style.display = "none";'
			. 'document.getElementById(\'submit-payment-form\').click();'
			. '</script>';
		die();
	}

	function call_webhooks()
	{

		$notification = new Notification(
			MODULE_PAYMENT_AXEPTA_SERVICE_ID,
			MODULE_PAYMENT_AXEPTA_SERVICE_KEY
		);

		$result_check_request_notification = $notification->checkRequest();

		if(is_int($result_check_request_notification)) {

			echo Notification::formatResponse(Notification::NS_ERROR);
			exit;
		}

		$currencies = Yii::$container->get('currencies');
		$order = $this->manager->getOrderInstanceWithId('\common\classes\Order', $result_check_request_notification['payment']['orderId']);

		if($order->order_id === 0 || (string) $order->info['order_status'] !== MODULE_PAYMENT_AXEPTA_ORDER_STATUS_ID) {
			echo Notification::formatResponse(Notification::NS_ERROR);
			exit;
		}

		$order_amount = round($currencies->format_clear($order->info['total_inc_tax'], true, $order->info['currency']), 2);

		$amount = Util::convertAmountToFractional(
			$order_amount
		);

		if(!$notification::checkRequestAmount(
			$result_check_request_notification,
			$amount,
			$order->info['currency'])) {

			echo Notification::formatResponse(Notification::NS_ERROR);
			exit;
		}

		$transaction_status = '';

		switch($result_check_request_notification['payment']['status']) {
			case Notification::PS_SETTLED:
				$notification_transaction_status = MODULE_PAYMENT_AXEPTA_SETTLED_ORDER_STATUS_ID;
				$transaction_status = 'OK';
				$payment_status = OrderPayment::OPYS_SUCCESSFUL;

				break;
			case Notification::PS_REJECTED:
				$notification_transaction_status = MODULE_PAYMENT_AXEPTA_REJECTED_ORDER_STATUS_ID;
				$payment_status = OrderPayment::OPYS_CANCELLED;

				break;
			default:
				echo Notification::formatResponse(Notification::NS_OK);
				exit;
		}

		$order_payment = new OrdersPayment();
		$order_payment->orders_payment_module = $this->code;
		$order_payment->orders_payment_module_name = $this->title;
		$order_payment->orders_payment_transaction_id = $result_check_request_notification['payment']['id'];
		$order_payment->orders_payment_id_parent = 0;
		$order_payment->orders_payment_order_id = $order->order_id;
		$order_payment->orders_payment_is_credit = 0;
		$order_payment->orders_payment_status = $payment_status;
		$order_payment->orders_payment_amount = $order_amount;
		$order_payment->orders_payment_currency = trim($order->info['currency']);
		$order_payment->orders_payment_currency_rate = (float) $order->info['currency_value'];
		$order_payment->orders_payment_snapshot = json_encode(OrderPayment::getOrderPaymentSnapshot($order));
		$order_payment->orders_payment_transaction_status = $transaction_status;
		$order_payment->orders_payment_transaction_commentary = '';
		$order_payment->orders_payment_date_create = date('Y-m-d H:i:s');
		$order_payment->orders_payment_transaction_full = $result_check_request_notification['payment']['status'];
		$order_payment->save();

		Order::setStatus($order->order_id, $notification_transaction_status, [
			'comments'          => '',
			'customer_notified' => 0,
		]);

		echo Notification::formatResponse(Notification::NS_OK);
	}

	function before_process()
	{

		return false;
	}

	function isOnline()
	{
		return true;
	}

	public function configure_keys()
	{

		$order_status_id = defined('MODULE_PAYMENT_AXEPTA_ORDER_STATUS_ID')
			? MODULE_PAYMENT_AXEPTA_ORDER_STATUS_ID
			: $this->getDefaultOrderStatusId();

		$order_status_settled_id = defined('MODULE_PAYMENT_AXEPTA_SETTLED_ORDER_STATUS_ID')
			? MODULE_PAYMENT_AXEPTA_SETTLED_ORDER_STATUS_ID
			: $this->getDefaultOrderStatusId();

		$order_status_rejected_id = defined('MODULE_PAYMENT_AXEPTA_REJECTED_ORDER_STATUS_ID')
			? MODULE_PAYMENT_AXEPTA_REJECTED_ORDER_STATUS_ID
			: $this->getDefaultOrderStatusId();

		return [
			'MODULE_PAYMENT_AXEPTA_STATUS'                   => [
				'title'        => 'Enable Axepta module',
				'value'        => 'True',
				'description'  => 'Do you want to enable Axepta module?',
				'sort_order'   => '1',
				'set_function' => 'tep_cfg_select_option(array(\'True\', \'False\'), ',
			],
			'MODULE_PAYMENT_AXEPTA_MERCHANT_ID'              => [
				'title'       => 'Merchant ID',
				'value'       => '',
				'description' => '',
				'sort_order'  => '2',
			],
			'MODULE_PAYMENT_AXEPTA_SERVICE_ID'               => [
				'title'       => 'Service ID',
				'value'       => '',
				'description' => '',
				'sort_order'  => '3',
			],
			'MODULE_PAYMENT_AXEPTA_SERVICE_KEY'              => [
				'title'       => 'Service Key',
				'value'       => '',
				'description' => '',
				'sort_order'  => '4',
			],
			'MODULE_PAYMENT_AXEPTA_CREDIT_TYPE'              => [
				'title'       => 'Credit type',
				'value'       => '',
				'description' => '',
				'sort_order'  => '5',
			],
			'MODULE_PAYMENT_AXEPTA_ZONE'                     => [
				'title'        => 'Payment zone',
				'value'        => '0',
				'description'  => 'If a zone is selected, only enable this payment method for that zone.',
				'sort_order'   => '6',
				'use_function' => '\\common\\helpers\\Zones::get_zone_class_title',
				'set_function' => 'tep_cfg_pull_down_zone_classes(',
			],
			'MODULE_PAYMENT_AXEPTA_ORDER_STATUS_ID'          => [
				'title'        => 'Set order status before payment',
				'description'  => 'Set the status of orders before redirect to Gateway',
				'value'        => $order_status_id,
				'use_function' => '\\common\\helpers\\Order::get_order_status_name',
				'set_function' => 'tep_cfg_pull_down_order_statuses(',
			],
			'MODULE_PAYMENT_AXEPTA_SETTLED_ORDER_STATUS_ID'  => [
				'title'        => 'Set order status settled payment',
				'description'  => 'Set the status of orders completed successfully',
				'value'        => $order_status_settled_id,
				'set_function' => 'tep_cfg_pull_down_order_statuses(',
				'use_function' => '\\common\\helpers\\Order::get_order_status_name',
			],
			'MODULE_PAYMENT_AXEPTA_REJECTED_ORDER_STATUS_ID' => [
				'title'        => 'Set order status rejected payment',
				'description'  => 'Set the status of orders completed successfully',
				'value'        => $order_status_rejected_id,
				'set_function' => 'tep_cfg_pull_down_order_statuses(',
				'use_function' => '\\common\\helpers\\Order::get_order_status_name',
			],
			'MODULE_PAYMENT_AXEPTA_SORT_ORDER'               => [
				'title'       => 'Sort order of display',
				'value'       => '0',
				'description' => 'Sort order of Custom payment display. Lowest is displayed first.',
				'sort_order'  => '8',
			],
		];
	}

	public function describe_status_key()
	{
		return new ModuleStatus('MODULE_PAYMENT_AXEPTA_STATUS', 'True', 'False');
	}

	public function describe_sort_key()
	{
		return new ModuleSortOrder('MODULE_PAYMENT_AXEPTA_SORT_ORDER');
	}

}